@@ -29,15 +29,15 @@ from signal import SIGTERM
 from rteval.modules import rtevalRuntimeError
 from rteval.modules.loads import CommandLineLoad
 from rteval.Log import Log
-from rteval.misc import expand_cpulist
+from rteval.misc import expand_cpulist,compress_cpulist
 from rteval.systopology import SysTopology
 
-kernel_prefix="linux-4.9"
+kernel_prefix="linux-5.1"
 
 class KBuildJob(object):
     '''Class to manage a build job bound to a particular node'''
 
-    def __init__(self, node, kdir, logger=None):
+    def __init__(self, node, kdir, logger=None, cpulist=None):
         self.kdir = kdir
         self.jobid = None
         self.node = node
@@ -46,11 +46,14 @@ class KBuildJob(object):
         self.objdir = "%s/node%d" % (self.builddir, int(node))
         if not os.path.isdir(self.objdir):
             os.mkdir(self.objdir)
-        if os.path.exists('/usr/bin/numactl'):
+        if os.path.exists('/usr/bin/numactl') and not cpulist:
             self.binder = 'numactl --cpunodebind %d' % int(self.node)
         else:
-            self.binder = 'taskset -c %s' % str(self.node)
-        self.jobs = self.calc_jobs_per_cpu() * len(self.node)
+            self.binder = 'taskset -c %s' % compress_cpulist(cpulist)
+        if cpulist:
+            self.jobs = self.calc_jobs_per_cpu() * len(cpulist)
+        else:
+            self.jobs = self.calc_jobs_per_cpu() * len(self.node)
         self.log(Log.DEBUG, "node %d: jobs == %d" % (int(node), self.jobs))
         self.runcmd = "%s make O=%s -C %s -j%d bzImage modules" % (self.binder, self.objdir, self.kdir, self.jobs)
         self.cleancmd = "%s make O=%s -C %s clean allmodconfig" % (self.binder, self.objdir, self.kdir)
@@ -105,12 +108,40 @@ class Kcompile(CommandLineLoad):
         self.buildjobs = {}
         self.config = config
         self.topology = SysTopology()
+        self.cpulist = config.cpulist
         CommandLineLoad.__init__(self, "kcompile", config, logger)
         self.logger = logger
 
+    def _extract_tarball(self):
+        if self.source == None:
+            raise rtevalRuntimeError(self, " no source tarball specified!")
+        self._log(Log.DEBUG, "unpacking kernel tarball")
+        tarargs = ['tar', '-C', self.builddir, '-x']
+        if self.source.endswith(".bz2"):
+            tarargs.append("-j")
+        elif self.source.endswith(".gz"):
+            tarargs.append("-z")
+        tarargs.append("-f")
+        tarargs.append(self.source)
+        try:
+            subprocess.call(tarargs)
+        except:
+            self._log(Log.DEBUG, "untar'ing kernel self.source failed!")
+            sys.exit(-1)
+
+    def _remove_build_dirs(self):
+        if not os.path.isdir(self.builddir):
+            return
+        self._log(Log.DEBUG, "removing kcompile directories in %s" % self.builddir)
+        null = os.open("/dev/null", os.O_RDWR)
+        cmd=["rm",  "-rf", os.path.join(self.builddir, "kernel*"), os.path.join(self.builddir, "node*")]
+        ret = subprocess.call(cmd,stdin=null, stdout=null, stderr=null)
+        if ret:
+            raise rtevalRuntimeError(self, "error removing builddir (%s) (ret=%d)" % (self.builddir, ret))
+
     def _WorkloadSetup(self):
         # find our source tarball
-        if self._cfg.has_key('tarball'):
+        if 'tarball' in self._cfg:
             tarfile = os.path.join(self.srcdir, self._cfg.tarfile)
             if not os.path.exists(tarfile):
                 raise rtevalRuntimeError(self, " tarfile %s does not exist!" % tarfile)
@@ -130,19 +161,7 @@ class Kcompile(CommandLineLoad):
                 kdir=d
                 break
         if kdir == None:
-            self._log(Log.DEBUG, "unpacking kernel tarball")
-            tarargs = ['tar', '-C', self.builddir, '-x']
-            if self.source.endswith(".bz2"):
-                tarargs.append("-j")
-            elif self.source.endswith(".gz"):
-                tarargs.append("-z")
-            tarargs.append("-f")
-            tarargs.append(self.source)
-            try:
-                subprocess.call(tarargs)
-            except:
-                self._log(Log.DEBUG, "untar'ing kernel self.source failed!")
-                sys.exit(-1)
+            self._extract_tarball()
             names = os.listdir(self.builddir)
             for d in names:
                 self._log(Log.DEBUG, "checking %s" % d)
@@ -156,9 +175,27 @@ class Kcompile(CommandLineLoad):
         self._log(Log.DEBUG, "systopology: %s" % self.topology)
         self.jobs = len(self.topology)
         self.args = []
-        for n in self.topology:
+
+        # get the cpus for each node
+        self.cpus = {}
+        self.nodes = self.topology.getnodes()
+        for n in self.nodes:
+            self.cpus[n] = [ int(c.split('/')[-1][3:]) for c in glob.glob('/sys/devices/system/node/node%s/cpu[0-9]*' % n) ]
+            self.cpus[n].sort()
+
+            # if a cpulist was specified, only allow cpus in that list on the node
+            if self.cpulist:
+                self.cpus[n] = [ c for c in self.cpus[n] if str(c) in expand_cpulist(self.cpulist) ]
+
+        # remove nodes with no cpus available for running
+        for node,cpus in self.cpus.items():
+            if not cpus:
+                self.nodes.remove(node)
+                self._log(Log.DEBUG, "node %s has no available cpus, removing" % node)
+
+        for n in self.nodes:
             self._log(Log.DEBUG, "Configuring build job for node %d" % int(n))
-            self.buildjobs[n] = KBuildJob(n, self.mydir, self.logger)
+            self.buildjobs[n] = KBuildJob(self.topology[n], self.mydir, self.logger, self.cpus[n] if self.cpulist else None)
             self.args.append(str(self.buildjobs[n])+";")
 
 
@@ -172,11 +209,18 @@ class Kcompile(CommandLineLoad):
 
         # clean up any damage from previous runs
         try:
-            ret = subprocess.call(["make", "-C", self.mydir, "mrproper"],
-                                  stdin=null, stdout=out, stderr=err)
+            cmd = ["make", "-C", self.mydir, "mrproper"]
+            ret = subprocess.call(cmd, stdin=null, stdout=out, stderr=err)
             if ret:
-                raise rtevalRuntimeError(self, "kcompile setup failed: %d" % ret)
-        except KeyboardInterrupt, m:
+                # if the above make failed, remove and reinstall the source tree
+                self._log(Log.DEBUG, "Invalid state in kernel build tree, reloading")
+                self._remove_build_dirs()
+                self._extract_tarball()
+                ret = subprocess.call(cmd, stdin=null, stdout=out, stderr=err)
+                if ret:
+                    # give up
+                    raise rtevalRuntimeError(self, "kcompile setup failed: %d" % ret)
+        except KeyboardInterrupt as m:
             self._log(Log.DEBUG, "keyboard interrupt, aborting")
             return
         self._log(Log.DEBUG, "ready to run")
@@ -184,7 +228,7 @@ class Kcompile(CommandLineLoad):
             os.close(out)
             os.close(err)
         # clean up object dirs and make sure each has a config file
-        for n in self.topology:
+        for n in self.nodes:
             self.buildjobs[n].clean(sin=null,sout=null,serr=null)
         os.close(null)
         self._setReady()
@@ -197,25 +241,36 @@ class Kcompile(CommandLineLoad):
         else:
             self.__outfd = self.__errfd = self.__nullfd
 
-        if self._cfg.has_key('cpulist') and self._cfg.cpulist:
+        if 'cpulist' in self._cfg and self._cfg.cpulist:
             cpulist = self._cfg.cpulist
             self.num_cpus = len(expand_cpulist(cpulist))
         else:
             cpulist = ""
 
     def _WorkloadTask(self):
-        for n in self.topology:
+        for n in self.nodes:
             if not self.buildjobs[n]:
-                raise RuntimeError, "Build job not set up for node %d" % int(n)
+                raise RuntimeError("Build job not set up for node %d" % int(n))
             if self.buildjobs[n].jobid is None or self.buildjobs[n].jobid.poll() is not None:
+                # A jobs was started, but now it finished. Check return code.
+                # -2 is returned when user forced stop of execution (CTRL-C).
+                if self.buildjobs[n].jobid is not None:
+                    if self.buildjobs[n].jobid.returncode != 0 and self.buildjobs[n].jobid.returncode != -2:
+                        raise RuntimeError("kcompile module failed to run (returned %d), please check logs for more detail"
+                                % self.buildjobs[n].jobid.returncode)
                 self._log(Log.INFO, "Starting load on node %d" % n)
                 self.buildjobs[n].run(self.__nullfd, self.__outfd, self.__errfd)
 
     def WorkloadAlive(self):
         # if any of the jobs has stopped, return False
-        for n in self.topology:
+        for n in self.nodes:
             if self.buildjobs[n].jobid.poll() is not None:
+                # Check return code (see above).
+                if self.buildjobs[n].jobid.returncode != 0 and self.buildjobs[n].jobid.returncode != -2:
+                    raise RuntimeError("kcompile module failed to run (returned %d), please check logs for more detail"
+                            % self.buildjobs[n].jobid.returncode)
                 return False
+
         return True
 
 
@@ -239,7 +294,7 @@ class Kcompile(CommandLineLoad):
 
 def ModuleParameters():
     return {"source":   {"descr": "Source tar ball",
-                         "default": "linux-4.9.tar.xz",
+                         "default": "linux-5.1.tar.xz",
                          "metavar": "TARBALL"},
             "jobspercore": {"descr": "Number of working threads per core",
                             "default": 2,
